@@ -26,9 +26,7 @@
 #	define MAP_FAILED ((void *)-1)
 #endif
 
-
 /* Implementational detail - not for global header */
-
 #define OLE_DEBUG 0
 
 /* FIXME tenix add ADD_BBD_LIST_BLOCK where it should be used) */
@@ -169,7 +167,8 @@ take_wrapper_functions (MsOle *f, MsOleSysWrappers *wrappers) {
 }
 
 
-/* A global variable to enable calles to check_stream,
+/*
+ * A global variable to enable calles to check_stream,
  * applications should optionally enable due to the performance penalty.
  * of 30-50 % of load time.
  */
@@ -557,14 +556,26 @@ ms_ole_debug (MsOle *fs, int magic)
 	}
 }
 
+/*
+ * get_next_block:
+ * @f:   the file handle
+ * @blk: an index into the big block fat
+ * 
+ * Return value: the block index of the BBD block.
+ */
 static BLP
-get_next_block (MsOle *f, BLP blk)
+get_next_block (MsOle *f, BLP blk, gboolean *err)
 {
-	/* blk is an index of the complete fat, and
-	   an index of the complete fat is the number of a BBD block */
-	BLP bbd     = GET_BBD_LIST (f, blk/(BB_BLOCK_SIZE/4));
-	return        MS_OLE_GET_GUINT32 (BB_R_PTR(f,bbd)
-					  + 4*(blk%(BB_BLOCK_SIZE/4)));
+	BLP bbd = GET_BBD_LIST (f, blk / (BB_BLOCK_SIZE / 4));
+
+	if (bbd > (f->length / BB_BLOCK_SIZE)) {
+		*err = TRUE;
+		return 0;
+	} else
+		*err = FALSE;
+	
+	return MS_OLE_GET_GUINT32 (BB_R_PTR (f, bbd) +
+				   4 * (blk % (BB_BLOCK_SIZE / 4)));
 }
 
 /* Builds the FAT */
@@ -601,7 +612,12 @@ read_bb (MsOle *f)
 	/* Add BBD's that live in the BBD list */
 	for (lp = 0; (lp < (f->length / BB_BLOCK_SIZE) - 1) &&
 		     (lp < MAX_SIZE_BBD_LIST * BB_BLOCK_SIZE / 4); lp++) {
-		tmp = get_next_block (f, lp);
+		gboolean err;
+
+		tmp = get_next_block (f, lp, &err);
+		if (err)
+			return 0;
+
 		g_array_append_val (f->bb, tmp);
 	}
 
